@@ -39,19 +39,33 @@ PyFortranObject_New(FortranDataDef* defs, f2py_void_func init) {
     int i;
     PyFortranObject *fp = NULL;
     PyObject *v = NULL;
-    if (init!=NULL)                           /* Initialize F90 module objects */
+    if (init!=NULL) {                        /* Initialize F90 module objects */
         (*(init))();
-    if ((fp = PyObject_New(PyFortranObject, &PyFortran_Type))==NULL) return NULL;
-    if ((fp->dict = PyDict_New())==NULL) return NULL;
+    }
+    fp = PyObject_New(PyFortranObject, &PyFortran_Type);
+    if (fp == NULL) {
+        return NULL;
+    }
+    if ((fp->dict = PyDict_New()) == NULL) {
+        Py_DECREF(fp);
+        return NULL;
+    }
     fp->len = 0;
-    while (defs[fp->len].name != NULL) fp->len++;
-    if (fp->len == 0) goto fail;
+    while (defs[fp->len].name != NULL) {
+        fp->len++;
+    }
+    if (fp->len == 0) {
+        goto fail;
+    }
     fp->defs = defs;
-    for (i=0;i<fp->len;i++)
+    for (i=0;i<fp->len;i++) {
         if (fp->defs[i].rank == -1) {                      /* Is Fortran routine */
             v = PyFortranObject_NewAsAttr(&(fp->defs[i]));
-            if (v==NULL) return NULL;
+            if (v==NULL) {
+                goto fail;
+            }
             PyDict_SetItemString(fp->dict,fp->defs[i].name,v);
+            Py_XDECREF(v);
         } else
             if ((fp->defs[i].data)!=NULL) { /* Is Fortran variable or array (not allocatable) */
                 if (fp->defs[i].type == NPY_STRING) {
@@ -65,13 +79,16 @@ PyFortranObject_New(FortranDataDef* defs, f2py_void_func init) {
                                     fp->defs[i].type, NULL, fp->defs[i].data, 0, NPY_ARRAY_FARRAY,
                                     NULL);
                 }
-                if (v==NULL) return NULL;
+                if (v==NULL) {
+                    goto fail;
+                }
                 PyDict_SetItemString(fp->dict,fp->defs[i].name,v);
+                Py_XDECREF(v);
             }
-    Py_XDECREF(v);
+    }
     return (PyObject *)fp;
  fail:
-    Py_XDECREF(v);
+    Py_XDECREF(fp);
     return NULL;
 }
 
@@ -80,7 +97,10 @@ PyFortranObject_NewAsAttr(FortranDataDef* defs) { /* used for calling F90 module
     PyFortranObject *fp = NULL;
     fp = PyObject_New(PyFortranObject, &PyFortran_Type);
     if (fp == NULL) return NULL;
-    if ((fp->dict = PyDict_New())==NULL) return NULL;
+    if ((fp->dict = PyDict_New())==NULL) {
+        PyObject_Del(fp);
+        return NULL;
+    }
     fp->len = 1;
     fp->defs = defs;
     return (PyObject *)fp;
@@ -91,18 +111,10 @@ PyFortranObject_NewAsAttr(FortranDataDef* defs) { /* used for calling F90 module
 static void
 fortran_dealloc(PyFortranObject *fp) {
     Py_XDECREF(fp->dict);
-    PyMem_Del(fp);
+    PyObject_Del(fp);
 }
 
 
-#if PY_VERSION_HEX >= 0x03000000
-#else
-static PyMethodDef fortran_methods[] = {
-    {NULL,          NULL}           /* sentinel */
-};
-#endif
-
-
 /* Returns number of bytes consumed from buf, or -1 on error. */
 static Py_ssize_t
 format_def(char *buf, Py_ssize_t size, FortranDataDef def)
@@ -135,7 +147,7 @@ format_def(char *buf, Py_ssize_t size, FortranDataDef def)
 
     if (def.data == NULL) {
         static const char notalloc[] = ", not allocated";
-        if (size < sizeof(notalloc)) {
+        if ((size_t) size < sizeof(notalloc)) {
             return -1;
         }
         memcpy(p, notalloc, sizeof(notalloc));
@@ -222,11 +234,7 @@ fortran_doc(FortranDataDef def)
     size--;
 
     /* p now points one beyond the last character of the string in buf */
-#if PY_VERSION_HEX >= 0x03000000
     s = PyUnicode_FromStringAndSize(buf, p - buf);
-#else
-    s = PyString_FromStringAndSize(buf, p - buf);
-#endif
 
     PyMem_Free(buf);
     return s;
@@ -241,7 +249,7 @@ fortran_doc(FortranDataDef def)
 
 static FortranDataDef *save_def; /* save pointer of an allocatable array */
 static void set_data(char *d,npy_intp *f) {  /* callback from Fortran */
-    if (*f)                               /* In fortran f=allocated(d) */
+    if (*f)                                  /* In fortran f=allocated(d) */
         save_def->data = d;
     else
         save_def->data = NULL;
@@ -252,8 +260,11 @@ static PyObject *
 fortran_getattr(PyFortranObject *fp, char *name) {
     int i,j,k,flag;
     if (fp->dict != NULL) {
-        PyObject *v = PyDict_GetItemString(fp->dict, name);
-        if (v != NULL) {
+        PyObject *v = _PyDict_GetItemStringWithError(fp->dict, name);
+        if (v == NULL && PyErr_Occurred()) {
+            return NULL;
+        }
+        else if (v != NULL) {
             Py_INCREF(v);
             return v;
         }
@@ -286,7 +297,6 @@ fortran_getattr(PyFortranObject *fp, char *name) {
         return fp->dict;
     }
     if (strcmp(name,"__doc__")==0) {
-#if PY_VERSION_HEX >= 0x03000000
         PyObject *s = PyUnicode_FromString(""), *s2, *s3;
         for (i=0;i<fp->len;i++) {
             s2 = fortran_doc(fp->defs[i]);
@@ -295,11 +305,6 @@ fortran_getattr(PyFortranObject *fp, char *name) {
             Py_DECREF(s);
             s = s3;
         }
-#else
-        PyObject *s = PyString_FromString("");
-        for (i=0;i<fp->len;i++)
-            PyString_ConcatAndDel(&s,fortran_doc(fp->defs[i]));
-#endif
         if (PyDict_SetItemString(fp->dict, name, s))
             return NULL;
         return s;
@@ -310,17 +315,11 @@ fortran_getattr(PyFortranObject *fp, char *name) {
             return NULL;
         return cobj;
     }
-#if PY_VERSION_HEX >= 0x03000000
-    if (1) {
-        PyObject *str, *ret;
-        str = PyUnicode_FromString(name);
-        ret = PyObject_GenericGetAttr((PyObject *)fp, str);
-        Py_DECREF(str);
-        return ret;
-    }
-#else
-    return Py_FindMethod(fortran_methods, (PyObject *)fp, name);
-#endif
+    PyObject *str, *ret;
+    str = PyUnicode_FromString(name);
+    ret = PyObject_GenericGetAttr((PyObject *)fp, str);
+    Py_DECREF(str);
+    return ret;
 }
 
 static int
@@ -414,48 +413,26 @@ fortran_repr(PyFortranObject *fp)
     PyObject *name = NULL, *repr = NULL;
     name = PyObject_GetAttrString((PyObject *)fp, "__name__");
     PyErr_Clear();
-#if PY_VERSION_HEX >= 0x03000000
     if (name != NULL && PyUnicode_Check(name)) {
         repr = PyUnicode_FromFormat("<fortran %U>", name);
     }
     else {
         repr = PyUnicode_FromString("<fortran object>");
     }
-#else
-    if (name != NULL && PyString_Check(name)) {
-        repr = PyString_FromFormat("<fortran %s>", PyString_AsString(name));
-    }
-    else {
-        repr = PyString_FromString("<fortran object>");
-    }
-#endif
     Py_XDECREF(name);
     return repr;
 }
 
 
 PyTypeObject PyFortran_Type = {
-#if PY_VERSION_HEX >= 0x03000000
     PyVarObject_HEAD_INIT(NULL, 0)
-#else
-    PyObject_HEAD_INIT(0)
-    0,                    /*ob_size*/
-#endif
-    "fortran",                    /*tp_name*/
-    sizeof(PyFortranObject),      /*tp_basicsize*/
-    0,                    /*tp_itemsize*/
-    /* methods */
-    (destructor)fortran_dealloc, /*tp_dealloc*/
-    0,                    /*tp_print*/
-    (getattrfunc)fortran_getattr, /*tp_getattr*/
-    (setattrfunc)fortran_setattr, /*tp_setattr*/
-    0,                    /*tp_compare/tp_reserved*/
-    (reprfunc)fortran_repr, /*tp_repr*/
-    0,                    /*tp_as_number*/
-    0,                    /*tp_as_sequence*/
-    0,                    /*tp_as_mapping*/
-    0,                    /*tp_hash*/
-    (ternaryfunc)fortran_call,                    /*tp_call*/
+    .tp_name ="fortran",
+    .tp_basicsize = sizeof(PyFortranObject),
+    .tp_dealloc = (destructor)fortran_dealloc,
+    .tp_getattr = (getattrfunc)fortran_getattr,
+    .tp_setattr = (setattrfunc)fortran_setattr,
+    .tp_repr = (reprfunc)fortran_repr,
+    .tp_call = (ternaryfunc)fortran_call,
 };
 
 /************************* f2py_report_atexit *******************************/
@@ -539,7 +516,7 @@ void f2py_report_on_exit(int exit_flag,void *name) {
     fprintf(stderr,"(d) f2py call-back interface, %6d calls  : %8d msec\n",
             cb_passed_counter,cb_passed_time);
 
-    fprintf(stderr,"(e) wrapped (Fortran/C) functions (acctual) : %8d msec\n\n",
+    fprintf(stderr,"(e) wrapped (Fortran/C) functions (actual) : %8d msec\n\n",
             passed_call_time-cb_passed_call_time-cb_passed_time);
     fprintf(stderr,"Use -DF2PY_REPORT_ATEXIT_DISABLE to disable this message.\n");
     fprintf(stderr,"Exit status: %d\n",exit_flag);
@@ -576,7 +553,7 @@ static void f2py_report_on_array_copy_fromany(void) {
  *
  * Description:
  * ------------
- * Provides array_from_pyobj function that returns a contigious array
+ * Provides array_from_pyobj function that returns a contiguous array
  * object with the given dimensions and required storage order, either
  * in row-major (C) or column-major (Fortran) order. The function
  * array_from_pyobj is very flexible about its Python object argument
@@ -590,23 +567,23 @@ static void f2py_report_on_array_copy_fromany(void) {
  * $Id: fortranobject.c,v 1.52 2005/07/11 07:44:20 pearu Exp $
  */
 
+static int check_and_fix_dimensions(const PyArrayObject* arr,
+                                    const int rank,
+                                    npy_intp *dims);
+
 static int
-count_nonpos(const int rank,
-             const npy_intp *dims) {
+count_negative_dimensions(const int rank,
+                          const npy_intp *dims) {
     int i=0,r=0;
     while (i<rank) {
-        if (dims[i] <= 0) ++r;
+        if (dims[i] < 0) ++r;
         ++i;
     }
     return r;
 }
 
-static int check_and_fix_dimensions(const PyArrayObject* arr,
-                                    const int rank,
-                                    npy_intp *dims);
-
 #ifdef DEBUG_COPY_ND_ARRAY
-void dump_dims(int rank, npy_intp* dims) {
+void dump_dims(int rank, npy_intp const* dims) {
     int i;
     printf("[");
     for(i=0;i<rank;++i) {
@@ -656,17 +633,18 @@ PyArrayObject* array_from_pyobj(const int type_num,
                                 const int rank,
                                 const int intent,
                                 PyObject *obj) {
-    /* Note about reference counting
-       -----------------------------
-       If the caller returns the array to Python, it must be done with
-       Py_BuildValue("N",arr).
-       Otherwise, if obj!=arr then the caller must call Py_DECREF(arr).
-
-       Note on intent(cache,out,..)
-       ---------------------
-       Don't expect correct data when returning intent(cache) array.
-
-    */
+    /*
+     * Note about reference counting
+     *  -----------------------------
+     * If the caller returns the array to Python, it must be done with
+     * Py_BuildValue("N",arr).
+     * Otherwise, if obj!=arr then the caller must call Py_DECREF(arr).
+     *
+     * Note on intent(cache,out,..)
+     * ---------------------
+     * Don't expect correct data when returning intent(cache) array.
+     *
+     */
     char mess[200];
     PyArrayObject *arr = NULL;
     PyArray_Descr *descr;
@@ -678,7 +656,7 @@ PyArrayObject* array_from_pyobj(const int type_num,
         || ((intent & F2PY_OPTIONAL) && (obj==Py_None))
         ) {
         /* intent(cache), optional, intent(hide) */
-        if (count_nonpos(rank,dims)) {
+        if (count_negative_dimensions(rank,dims) > 0) {
             int i;
             strcpy(mess, "failed to create intent(cache|hide)|optional array"
                    "-- must have defined dimensions but got (");
@@ -719,8 +697,8 @@ PyArrayObject* array_from_pyobj(const int type_num,
             /* intent(cache) */
             if (PyArray_ISONESEGMENT(arr)
                 && PyArray_ITEMSIZE(arr)>=elsize) {
-                if (check_and_fix_dimensions(arr,rank,dims)) {
-                    return NULL; /*XXX: set exception */
+                if (check_and_fix_dimensions(arr, rank, dims)) {
+                    return NULL;
                 }
                 if (intent & F2PY_INTENT_OUT)
                     Py_INCREF(arr);
@@ -741,20 +719,20 @@ PyArrayObject* array_from_pyobj(const int type_num,
 
         /* here we have always intent(in) or intent(inout) or intent(inplace) */
 
-        if (check_and_fix_dimensions(arr,rank,dims)) {
-            return NULL; /*XXX: set exception */
+        if (check_and_fix_dimensions(arr, rank, dims)) {
+            return NULL;
         }
-	/*
-	printf("intent alignement=%d\n", F2PY_GET_ALIGNMENT(intent));
-	printf("alignement check=%d\n", F2PY_CHECK_ALIGNMENT(arr, intent));
-	int i;
-	for (i=1;i<=16;i++)
-	  printf("i=%d isaligned=%d\n", i, ARRAY_ISALIGNED(arr, i));
-	*/
+        /*
+        printf("intent alignment=%d\n", F2PY_GET_ALIGNMENT(intent));
+        printf("alignment check=%d\n", F2PY_CHECK_ALIGNMENT(arr, intent));
+        int i;
+        for (i=1;i<=16;i++)
+          printf("i=%d isaligned=%d\n", i, ARRAY_ISALIGNED(arr, i));
+        */
         if ((! (intent & F2PY_INTENT_COPY))
             && PyArray_ITEMSIZE(arr)==elsize
             && ARRAY_ISCOMPATIBLE(arr,type_num)
-	    && F2PY_CHECK_ALIGNMENT(arr, intent)
+            && F2PY_CHECK_ALIGNMENT(arr, intent)
             ) {
             if ((intent & F2PY_INTENT_C)?PyArray_ISCARRAY(arr):PyArray_ISFARRAY(arr)) {
                 if ((intent & F2PY_INTENT_OUT)) {
@@ -780,8 +758,8 @@ PyArrayObject* array_from_pyobj(const int type_num,
             if (!(ARRAY_ISCOMPATIBLE(arr,type_num)))
                 sprintf(mess+strlen(mess)," -- input '%c' not compatible to '%c'",
                         PyArray_DESCR(arr)->type,typechar);
-	    if (!(F2PY_CHECK_ALIGNMENT(arr, intent)))
-	      sprintf(mess+strlen(mess)," -- input not %d-aligned", F2PY_GET_ALIGNMENT(intent));
+            if (!(F2PY_CHECK_ALIGNMENT(arr, intent)))
+              sprintf(mess+strlen(mess)," -- input not %d-aligned", F2PY_GET_ALIGNMENT(intent));
             PyErr_SetString(PyExc_ValueError,mess);
             return NULL;
         }
@@ -818,9 +796,10 @@ PyArrayObject* array_from_pyobj(const int type_num,
     if ((intent & F2PY_INTENT_INOUT) ||
             (intent & F2PY_INTENT_INPLACE) ||
             (intent & F2PY_INTENT_CACHE)) {
-        PyErr_SetString(PyExc_TypeError,
-                        "failed to initialize intent(inout|inplace|cache) "
-                        "array, input not an array");
+        PyErr_Format(PyExc_TypeError,
+                     "failed to initialize intent(inout|inplace|cache) "
+                     "array, input '%s' object is not an array",
+                     Py_TYPE(obj)->tp_name);
         return NULL;
     }
 
@@ -842,8 +821,9 @@ PyArrayObject* array_from_pyobj(const int type_num,
                             | NPY_ARRAY_FORCECAST, NULL);
         if (arr==NULL)
             return NULL;
-        if (check_and_fix_dimensions(arr,rank,dims))
-            return NULL; /*XXX: set exception */
+        if (check_and_fix_dimensions(arr, rank, dims)) {
+            return NULL;
+        }
         return arr;
     }
 
@@ -854,12 +834,17 @@ PyArrayObject* array_from_pyobj(const int type_num,
 /*****************************************/
 
 static
-int check_and_fix_dimensions(const PyArrayObject* arr,const int rank,npy_intp *dims) {
+int check_and_fix_dimensions(const PyArrayObject* arr, const int rank, npy_intp *dims)
+{
     /*
-      This function fills in blanks (that are -1\'s) in dims list using
-      the dimensions from arr. It also checks that non-blank dims will
-      match with the corresponding values in arr dimensions.
-    */
+     * This function fills in blanks (that are -1's) in dims list using
+     * the dimensions from arr. It also checks that non-blank dims will
+     * match with the corresponding values in arr dimensions.
+     *
+     * Returns 0 if the function is successful.
+     *
+     * If an error condition is detected, an exception is set and 1 is returned.
+     */
     const npy_intp arr_size = (PyArray_NDIM(arr))?PyArray_Size((PyObject *)arr):1;
 #ifdef DEBUG_COPY_ND_ARRAY
     dump_attrs(arr);
@@ -876,9 +861,10 @@ int check_and_fix_dimensions(const PyArrayObject* arr,const int rank,npy_intp *d
             d = PyArray_DIM(arr,i);
             if (dims[i] >= 0) {
                 if (d>1 && dims[i]!=d) {
-                    fprintf(stderr,"%d-th dimension must be fixed to %" NPY_INTP_FMT
-                            " but got %" NPY_INTP_FMT "\n",
-                            i,dims[i], d);
+                    PyErr_Format(PyExc_ValueError,
+                                 "%d-th dimension must be fixed to %"
+                                 NPY_INTP_FMT " but got %" NPY_INTP_FMT "\n",
+                                 i, dims[i], d);
                     return 1;
                 }
                 if (!dims[i]) dims[i] = 1;
@@ -889,9 +875,10 @@ int check_and_fix_dimensions(const PyArrayObject* arr,const int rank,npy_intp *d
         }
         for(i=PyArray_NDIM(arr);i<rank;++i)
             if (dims[i]>1) {
-                fprintf(stderr,"%d-th dimension must be %" NPY_INTP_FMT
-                        " but got 0 (not defined).\n",
-                        i,dims[i]);
+                PyErr_Format(PyExc_ValueError,
+                             "%d-th dimension must be %" NPY_INTP_FMT
+                             " but got 0 (not defined).\n",
+                             i, dims[i]);
                 return 1;
             } else if (free_axe<0)
                 free_axe = i;
@@ -902,9 +889,11 @@ int check_and_fix_dimensions(const PyArrayObject* arr,const int rank,npy_intp *d
             new_size *= dims[free_axe];
         }
         if (new_size != arr_size) {
-            fprintf(stderr,"unexpected array size: new_size=%" NPY_INTP_FMT
-                    ", got array with arr_size=%" NPY_INTP_FMT " (maybe too many free"
-                    " indices)\n", new_size,arr_size);
+            PyErr_Format(PyExc_ValueError,
+                         "unexpected array size: new_size=%" NPY_INTP_FMT
+                         ", got array with arr_size=%" NPY_INTP_FMT
+                         " (maybe too many free indices)\n",
+                         new_size, arr_size);
             return 1;
         }
     } else if (rank==PyArray_NDIM(arr)) {
@@ -912,12 +901,13 @@ int check_and_fix_dimensions(const PyArrayObject* arr,const int rank,npy_intp *d
         int i;
         npy_intp d;
         for (i=0; i<rank; ++i) {
-	    d = PyArray_DIM(arr,i);
+            d = PyArray_DIM(arr,i);
             if (dims[i]>=0) {
                 if (d > 1 && d!=dims[i]) {
-                    fprintf(stderr,"%d-th dimension must be fixed to %" NPY_INTP_FMT
-                            " but got %" NPY_INTP_FMT "\n",
-                            i,dims[i],d);
+                    PyErr_Format(PyExc_ValueError,
+                                 "%d-th dimension must be fixed to %"
+                                 NPY_INTP_FMT " but got %" NPY_INTP_FMT "\n",
+                                 i, dims[i], d);
                     return 1;
                 }
                 if (!dims[i]) dims[i] = 1;
@@ -925,8 +915,10 @@ int check_and_fix_dimensions(const PyArrayObject* arr,const int rank,npy_intp *d
             new_size *= dims[i];
         }
         if (new_size != arr_size) {
-            fprintf(stderr,"unexpected array size: new_size=%" NPY_INTP_FMT
-                    ", got array with arr_size=%" NPY_INTP_FMT "\n", new_size,arr_size);
+            PyErr_Format(PyExc_ValueError,
+                         "unexpected array size: new_size=%" NPY_INTP_FMT
+                         ", got array with arr_size=%" NPY_INTP_FMT "\n",
+                         new_size, arr_size);
             return 1;
         }
     } else { /* [[1,2]] -> [[1],[2]] */
@@ -938,8 +930,10 @@ int check_and_fix_dimensions(const PyArrayObject* arr,const int rank,npy_intp *d
             if (PyArray_DIM(arr,i)>1) ++effrank;
         if (dims[rank-1]>=0)
             if (effrank>rank) {
-                fprintf(stderr,"too many axes: %d (effrank=%d), expected rank=%d\n",
-                        PyArray_NDIM(arr),effrank,rank);
+                PyErr_Format(PyExc_ValueError,
+                             "too many axes: %d (effrank=%d), "
+                             "expected rank=%d\n",
+                             PyArray_NDIM(arr), effrank, rank);
                 return 1;
             }
 
@@ -949,9 +943,11 @@ int check_and_fix_dimensions(const PyArrayObject* arr,const int rank,npy_intp *d
             else d = PyArray_DIM(arr,j++);
             if (dims[i]>=0) {
                 if (d>1 && d!=dims[i]) {
-                    fprintf(stderr,"%d-th dimension must be fixed to %" NPY_INTP_FMT
-                            " but got %" NPY_INTP_FMT " (real index=%d)\n",
-                            i,dims[i],d,j-1);
+                    PyErr_Format(PyExc_ValueError,
+                                 "%d-th dimension must be fixed to %"
+                                 NPY_INTP_FMT " but got %" NPY_INTP_FMT
+                                 " (real index=%d)\n",
+                                 i, dims[i], d, j-1);
                     return 1;
                 }
                 if (!dims[i]) dims[i] = 1;
@@ -967,13 +963,28 @@ int check_and_fix_dimensions(const PyArrayObject* arr,const int rank,npy_intp *d
         }
         for (i=0,size=1;i<rank;++i) size *= dims[i];
         if (size != arr_size) {
-            fprintf(stderr,"unexpected array size: size=%" NPY_INTP_FMT ", arr_size=%" NPY_INTP_FMT
-                    ", rank=%d, effrank=%d, arr.nd=%d, dims=[",
-                    size,arr_size,rank,effrank,PyArray_NDIM(arr));
-            for (i=0;i<rank;++i) fprintf(stderr," %" NPY_INTP_FMT,dims[i]);
-            fprintf(stderr," ], arr.dims=[");
-            for (i=0;i<PyArray_NDIM(arr);++i) fprintf(stderr," %" NPY_INTP_FMT,PyArray_DIM(arr,i));
-            fprintf(stderr," ]\n");
+            char msg[200];
+            int len;
+            snprintf(msg, sizeof(msg),
+                     "unexpected array size: size=%" NPY_INTP_FMT
+                     ", arr_size=%" NPY_INTP_FMT
+                     ", rank=%d, effrank=%d, arr.nd=%d, dims=[",
+                     size, arr_size, rank, effrank, PyArray_NDIM(arr));
+            for (i = 0; i < rank; ++i) {
+                len = strlen(msg);
+                snprintf(msg + len, sizeof(msg) - len,
+                         " %" NPY_INTP_FMT, dims[i]);
+            }
+            len = strlen(msg);
+            snprintf(msg + len, sizeof(msg) - len, " ], arr.dims=[");
+            for (i = 0; i < PyArray_NDIM(arr); ++i) {
+                len = strlen(msg);
+                snprintf(msg + len, sizeof(msg) - len,
+                         " %" NPY_INTP_FMT, PyArray_DIM(arr, i));
+            }
+            len = strlen(msg);
+            snprintf(msg + len, sizeof(msg) - len, " ]\n");
+            PyErr_SetString(PyExc_ValueError, msg);
             return 1;
         }
     }
@@ -999,8 +1010,6 @@ int copy_ND_array(const PyArrayObject *arr, PyArrayObject *out)
 /* Compatibility functions for Python >= 3.0 */
 /*********************************************/
 
-#if PY_VERSION_HEX >= 0x03000000
-
 PyObject *
 F2PyCapsule_FromVoidPtr(void *ptr, void (*dtor)(PyObject *))
 {
@@ -1027,29 +1036,6 @@ F2PyCapsule_Check(PyObject *ptr)
     return PyCapsule_CheckExact(ptr);
 }
 
-#else
-
-PyObject *
-F2PyCapsule_FromVoidPtr(void *ptr, void (*dtor)(void *))
-{
-    return PyCObject_FromVoidPtr(ptr, dtor);
-}
-
-void *
-F2PyCapsule_AsVoidPtr(PyObject *ptr)
-{
-    return PyCObject_AsVoidPtr(ptr);
-}
-
-int
-F2PyCapsule_Check(PyObject *ptr)
-{
-    return PyCObject_Check(ptr);
-}
-
-#endif
-
-
 #ifdef __cplusplus
 }
 #endif
